import sys
from PyQt4 import QtGui
from PyQt4 import QtCore
from PyQt4.QtCore import Qt
import os.path
import codecs
import osgqt
import osgDB
import osg
import shutil
import jarvis

class MyTextEdit(QtGui.QTextEdit):
    def __init__(self, text, father):
        super(MyTextEdit, self).__init__(text, father)
#    def keyPressEvent(self, event):
#        print event
    
class JarvisMain(QtGui.QWidget):
    
    def __init__(self, layout=None):
        self.osg_enable = True
        super(JarvisMain, self).__init__()
        
        self.initUI(layout)

    def message(self, object):
        fun_name = object["fun"]
        args = object.get("args", [])
        kwargs = object.get("kwargs", {})
        getattr(self, fun_name)(*args, **kwargs)

    def start(self):
        self.debug.clear()
        self.error.clear()

    def finish(self):
        pass
    
    def display(self):
        print self.editor.toPlainText()

#    def wheelEvent(self, event):
#        print self.editor.setFocus(False)

#    def keyPressEvent(self, event):
#         print event

    def createEditor(self, text, width, height):
        editor = MyTextEdit(text, self)
        editor.setMinimumWidth(width)
        editor.setMinimumHeight(height)
        return editor

    def createOSG(self, width, height):        
        osgWidget = osgqt.PyQtOSGWidget(self)
        osgWidget.setMinimumWidth(width)
        osgWidget.setMinimumHeight(height)
        return osgWidget

    def parse_layout(self, layout):
        ret = {}
        f = open(layout)
        for line in f:
            parts = line.split("=")
            ret[parts[0]] = int(parts[1])
        return ret
                
    def initUI(self, layout=None):
        self.setWindowTitle('Jarvis')

        if layout is not None:
            layout = self.parse_layout(layout)
        else:
            layout = {}

        print layout

        WIDTH=480
        HEIGHT=1024
        self.setGeometry(1440 - WIDTH, 0, WIDTH, 1024)

#        self.setWindowIcon(QtGui.QIcon('web.png'))

        text = ""
#        for i in range(10):
#            text += '<b>Text%d</b><br/>' % i

        self.topBox = QtGui.QHBoxLayout(self)
        self.topBox.setContentsMargins(0,0,0,0)
        self.topBox.setSpacing(0)
        self.topBox.setGeometry(QtCore.QRect(0, 0, WIDTH, HEIGHT))
  
#        self.editor = self.createEditor(text, 300, 600)
#        self.editor.setSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Expanding)
#        self.topBox.addWidget(self.editor)

        self.rightBox = QtGui.QVBoxLayout()
        self.rightBox.setSpacing(0)
        self.topBox.addLayout(self.rightBox)


        self.error = self.createEditor("", WIDTH, 200)
        self.debug = self.createEditor("", WIDTH, 200)
        
        if self.osg_enable:
            self.osgView = self.createOSG(WIDTH, int(WIDTH * 9 / 16.0))

        self.rightBox.addWidget(self.error, 0, Qt.AlignRight)
        self.rightBox.addWidget(self.debug, 0, Qt.AlignRight)
        if self.osg_enable:
            self.rightBox.addWidget(self.osgView, 0, Qt.AlignRight)

#        editor.textChanged.connect(self.display)
#        editor.wheelEvent.connect(self.wheelEvent)
#        self.editor.setFocus()
        self.filename = None
        self.show()

    def atomic_write(self, filename, text):
        f = open(filename + ".tmp", "w")
        f.write(text)
        f.close()
        shutil.move(filename + ".tmp", filename)        

    def debugprint(self, *args):
        text = " ".join(map(lambda x: str(x), args)) + "\n"
        self.debug.append(text)
        text = self.debug.toPlainText()

        debug_file = jarvis.get_filename(jarvis.DEBUG_FILE)
        self.atomic_write(debug_file, text)
        
    def errorprint(self, *args):
        text = " ".join(map(lambda x: str(x), args)) + "\n"
        self.error.append(text)
        text = self.error.toPlainText()

        error_file = jarvis.get_filename(jarvis.ERROR_FILE)
        self.atomic_write(error_file, text)

    def osgprint(self, data):
        self.osgView.setSceneData(data)

    def setlooptime(self, loopTime):
        self.osgView.setLoopTime(loopTime)

    def runcommand(self, fun):
        fun()

    def getosgviewer(self):
        return self.osgView.getosgviewer()

    def file_dialog(self):
        fd = QtGui.QFileDialog(self)
        filename = fd.getOpenFileName()
        if os.path.isfile(filename):
            text = open(filename).read()
            self.editor.setText(text)
            self.filename = filename

            s = codecs.open(self.filename,'w','utf-8')
            s.write(unicode(self.ui.editor_window.toPlainText()))
            s.close()



def main():    
    app = QtGui.QApplication(sys.argv)
    
    ex = JarvisMain()
#    ex.raise_()


    # create a root node
    node = osg.Group()
    # open a file
    loadedmodel = osgDB.readNodeFile("/Users/lagunas/tmp/osgIphoneExampleGLES2/cow.osg")
    node.addChild(loadedmodel)
    
    ex.osgView.viewer.setSceneData(node)

    sys.exit(app.exec_())

if __name__ == '__main__':
    main()    



import os
import argparse
import traceback

ml = None

# This is the place where is stored the currently tested 'module.function' .
TEST_MODULE_PATH="testmodule.txt"

# This is the file where errors produced by calls to jarvis.command.error are written.
ERROR_FILE="error.txt"
DEBUG_FILE="debug.txt"

def get_home():
    jarvis_home = os.getenv("JARVIS_HOME", "/tmp/jarvis")
    try:
        os.makedirs(jarvis_home)
    except OSError, e:
        if e.errno != 17:
            raise
        
    return jarvis_home

def get_filename(key):
    error_file = os.path.join(get_home(), key)    
    return error_file

def run(modulename, layout=None):
    global ml
    import qtdisplay
    import mainloop

    try:
        ml = mainloop.MainLoop(modulename)
        
        display = qtdisplay.QTDisplay(ml, layout)
        ml.setdisplay(display)

        display.init()
        
    except KeyboardInterrupt:
        pass
    except Exception, e:
        print traceback.format_exc(e)
    finally:
        display.destroy()
        

def main():
    parser = argparse.ArgumentParser(description='Process some integers.')
    parser.add_argument("--nogl", default=False, action="store_true", help="disable gl window")
    parser.add_argument("--module", metavar="NAME")
    parser.add_argument("--layout", metavar="LAYOUT", default=None)

    args = parser.parse_args()
    run(args.module, args.layout)

if __name__ == "__main__":
    main()

from commands import *

def coucou():
    a = 4



import os
import os.path
import sys
import time
import traceback
from PyQt4 import QtCore
import jarvis
import rollbackimporter

class Display():
    def __init__(self):
        pass

    def init(self):
        pass

    def destroy(self):
        pass

    def start(self):
        pass
    
    def finish(self):
        pass

    def debugprint(self, *args):
        info = " ".join(map(lambda x: str(x), args)) + "\n"
        print "INFO", info

    def errorprint(self, *args):
        info = " ".join(map(lambda x: str(x), args)) + "\n"
        print "INFO", info

        
class MainLoop(QtCore.QThread):
    def __init__(self, module, display = None):
        QtCore.QThread.__init__(self)

        self.module_function_name = module
        self.finished = False
        self.display = display
        self.filedates = {}
        self.module = None

        modName = self.module_function_name
        modNameParts = modName.split(".")
        self.module_name = ".".join(modNameParts[:-1])
        
        self.rollbackImporter = rollbackimporter.RollbackImporter()
        self.watchfiles = {}
        # This flag says if last test was runned or not yet finished
        self.run_finished = True

    def add_watch_file(self, filename):
        self.watchfiles[filename] = True

    def setdisplay(self, display):
        self.display = display

    def checkreloadfile(self, filename):
#        debug = "lizard" in filename
#        if debug:
#            print filename
        modified = False
        try:
            filedate = os.path.getmtime(filename)
            self.watchfiles[filename] = True
            checkedfile = filename
        except OSError:
            return False, None

        lastdate = self.filedates.get(filename)

#        if debug:
#            print "lastdate", lastdate, "filedate", filedate
            
        if lastdate == None:
            lastdate = filedate
        elif lastdate < filedate:
            lastdate = filedate
            modified = True
#            print "MODIFIED"
            
        self.filedates[filename] = lastdate            

        return modified, checkedfile

    def checkreloadmodule(self, module):
        checkedfiles = []
        if not hasattr(module, "__file__"):
            return False, []
        
        filename = module.__file__

        if filename == None:
            return False, []
#        if "common" in filename:
#        print "checkreloadmodule", filename

        if filename.endswith(".pyc"):
            filename = filename[:-1]

#        if "test_" in filename:
#            print "HERE !!"

        modified = False
        for suffix in [""]:
            localModified, checkedfile = self.checkreloadfile(filename + suffix)
            modified = modified or localModified
            if checkedfile != None:
                checkedfiles += [checkedfile]
                            
        return modified, checkedfiles

    def loadMainModule(self):
        modName = self.module_function_name
        modNameParts = modName.split(".")
        mod = __import__(".".join(modNameParts[:-1]))

#        print "FOUND", "test_" in sys.modules.keys()
        for p in modNameParts[1:-1]:            
            mod = getattr(mod, p)
        self.module = mod
#        reload(self.module)

    def modulechanged(self):
        # Check that main module has not changed
        testmodulepath = os.path.join(jarvis.get_home(), jarvis.TEST_MODULE_PATH)
        testmodulename = None
        
        testmodulename = open(testmodulepath).read()
        if testmodulename != self.module_function_name:
            self.module_function_name = testmodulename
            print "CHANGED", testmodulename
            return True
        return False

    def singlePrepare(self):
        
        modified = False
        self.daemon = True
        # Check if any file was modified since last run

        modules = sys.modules.values()

#        print "is there", "test_" in sys.modules.keys()
#        print "\n".join(sys.modules.keys())
        fullcheckedfiles = {}
                
        for m in modules:
            if m != None:
                try:
                    checkreload, checkedfiles = self.checkreloadmodule(m)
                    for filename in checkedfiles:
                        fullcheckedfiles[filename] = True
                    modified = modified or checkreload
                    if modified:
                        break
                except Exception, e:                    
#                    pass
                    print traceback.format_exc(e)
                    print e.__class__.__name__

        for filename in self.watchfiles:
#            if "test_" in filename:
#                print filename, filename in fullcheckedfiles
            
            if filename not in fullcheckedfiles:
                checkreload, checkedfile = self.checkreloadfile(filename)
                if checkreload:
                    modified = True
                    break
                    

        return modified

    def runcommand(self):
        fun = getattr(self.module, self.module_function_name.split(".")[-1])
        try:
            fun()
        except Exception, e:
            exc_type, exc_obj, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            self.display.errorprint("%s:%s\n" % (fname, exc_tb.tb_lineno) + traceback.format_exc(e) + "\n")
        finally:
            self.run_finished = True
        
    def singleRun(self):
        if self.display != None:
            self.display.runcommand(self.runcommand)

    def runloop(self):
        if not self.run_finished:
            return

        if self.module == None:
            first = True
        else:
            first = False

        modified = self.modulechanged()
            
        self.loadMainModule()

        modified = modified or self.singlePrepare()
        
        if not first:
            if not modified:                
                return

        self.run_finished = False

        self.display.start()

        if self.rollbackImporter:
            self.rollbackImporter.cleanup(self.display)


        self.singleRun()
                
    def run_(self):
        while(not self.finished):
            try:
                self.runloop()
            except Exception, e:
                if self.display != None:
                    self.display.start()
                    self.display.errorprint(traceback.format_exc(e))
                    self.display.finish()
            time.sleep(0.5)
                                                
    def run(self):
        # Write the current module name to disk
        testmodulepath = os.path.join(jarvis.get_home(), jarvis.TEST_MODULE_PATH)
        f = open(testmodulepath, "w")
        f.write(self.module_function_name)
        f.close()        

        try:
            self.run_()
        except KeyboardInterrupt:
            pass
        except Exception, e:
            print e
        





import os.path
import jinja2
import imp
import time
import jarvis.emacs as emacs

dirname = os.path.dirname(os.path.abspath(__file__))
snippetdirnames = []

snippetdirnames = emacs.get_dirs("snippets")

def get_snippet_dir_names():
    return snippetdirnames
    
def load_snippet(commandname, **kwargs):
    for snippetdirname in snippetdirnames:        
        commandfilename = os.path.join(snippetdirname, commandname + ".py")
        if os.path.exists(commandfilename):
            break
    s = open(commandfilename).read()
    
    template = jinja2.Template(s)
    
    s = template.render(**kwargs)

    return s


# Find the "tests" directory
def find_tests_path(filename):
    new_directory = filename
    test_path = None
    while(True):
        old_directory = new_directory
        test_path_try = os.path.join(new_directory, "tests")
        if os.path.exists(test_path_try) and os.path.isdir(test_path_try):
            test_path = test_path_try
            break

        new_directory = os.path.abspath(os.path.join(new_directory, ".."))

        if new_directory == old_directory:
            break
    return test_path


def find_module_name(filename):
    basename = os.path.basename(filename) 
    if basename == "__init__.py":
        module_name = []
    else:
        module_name = [basename[:-3]]

    current_dir = os.path.dirname(filename)
        
    while(True):
        old_current_dir = current_dir
        initfile = os.path.join(current_dir, "__init__.py")
        if os.path.exists(initfile):
            module_name = [os.path.basename(current_dir)] + module_name            
        else:
            break

        current_dir = os.path.abspath(os.path.join(current_dir, ".."))

        if current_dir == old_current_dir:
            break
    return ".".join(module_name)

def load_module_by_filename(filename):
    return imp.load_source("entrypoint" + str(time.time()).replace(".", "_"), filename)

def create_init_files(filename):
    current_dir = os.path.dirname(filename)
        
    while(True):
        old_current_dir = current_dir
        initfile = os.path.join(current_dir, "__init__.py")
        if os.path.exists(initfile):
            break
        else:
            f = open(initfile, "w")
            f.close()

        current_dir = os.path.abspath(os.path.join(current_dir, ".."))

        if current_dir == old_current_dir:
            break

def module_load_source(filename):
    import imp
    name = "entrypoint" + str(time.time()).replace(".", "_")

    pyc_filename = filename + "c"
    if os.path.exists(pyc_filename) and filename.endswith(".py"):
        os.remove(pyc_filename)

    mod = imp.load_source(name, filename)
    return mod
        
def find_class_names(filename):
    import inspect
    
    class_names = []
    # Load the module
    f = module_load_source(filename) 

    # Enumerate the classes in the module
    for k in dir(f):
        # Get the task class
        cl = getattr(f, k)
        if inspect.isclass(cl):
            class_names  += [cl.__name__]

    return class_names



#!/usr/bin/env python
#osgviewerQt4.py

__author__  = ["Rene Molenaar"]
__url__     = ("http://code.google.com/p/osgswig/")
__version__ = "1.0.0"
__doc__     = """ This example shows to use osgViewer.Viewer within PyQt4 ____Rene Molenaar 2008 
                  the basics functionality is present, but there are many 'missing features'
                  for example, mouse modifiers are not adapted, mouse wheel is not adapted and 
                  non-ascii keyboard input is not adapted.
              """

# general Python
import sys,os

import OpenGL.GL as GL

# import qtWidgets stuff
import PyQt4

# import OpenSceneGraph wrapper
import osg, osgDB, osgGA, osgViewer

from PyQt4 import QtOpenGL
from PyQt4 import Qt, QtGui, QtCore
import time

mouseButtonDictionary = {
    QtCore.Qt.LeftButton: 1,
    QtCore.Qt.MidButton: 2,
    QtCore.Qt.RightButton: 3,
    QtCore.Qt.NoButton: 0,
    }

class PyQtOSGWidget(QtOpenGL.QGLWidget):
    def __init__(self, parent = 0, name = '' ,flags = 0):
        """constructor """
        QtOpenGL.QGLWidget.__init__(self, parent)
        self.setFocusPolicy(QtCore.Qt.ClickFocus)
        self.timer = Qt.QTimer()
        self.timer.setInterval(40)
        self.camera = None
        self.startTime = time.time()
        self.loopTime = 10.0

    def initializeGL (self):
        """initializeGL the context and create the osgViewer, also set manipulator and event handler """
        self.gw = self.createContext()
        self.viewer = self.createViewer()
        #init the default eventhandler
#        self.viewer.setCameraManipulator(osgGA.TrackballManipulator())
        self.viewer.addEventHandler(osgViewer.StatsHandler())
        self.viewer.addEventHandler(osgViewer.HelpHandler())
        QtCore.QObject.connect(self.timer, QtCore.SIGNAL("timeout ()"), self.updateGL)
        self.timer.start(40)

    def setLoopTime(self, loopTime):
        self.loopTime = loopTime

    def embedInContext (self):
        """create a osg.GraphicsWindow for a Qt.QWidget window"""
        gw = osgViewer.GraphicsWindowEmbedded(0,0,self.width(),self.height())
        return gw

    def createContext (self):
        """create a opengl context (currently WindowData classes are not wrapped so we can not inherrit the windowdata) """
        ds = osg.DisplaySettings_instance()
        if False:
            traits = osg.GraphicsContext.Traits()
            print traits
            traits.readDISPLAY()
            if (traits.displayNum<0): traits.displayNum = 0

            traits.windowName = "osgViewerPyQt"
            traits.screenNum = 0
            traits.x = self.x()
            traits.y = self.y()
            traits.width = self.width()
            traits.height = self.height()
            traits.alpha = 8 #ds.getMinimumNumAlphaBits()
            traits.stencil = 8 #ds.getMinimumNumStencilBits()
            traits.windowDecoration = False
            traits.doubleBuffer = True
            traits.sampleBuffers = 4 #ds.getMultiSamples()
            traits.samples = 4 #ds.getNumMultiSamples()
        gw = osgViewer.GraphicsWindowEmbedded()
        return gw

    def createViewer(self):
        """create a osgViewer.Viewer and set the viewport, camera and previously created graphical context """
        viewer = osgViewer.Viewer()
        self.resetCamera(viewer)
        return viewer

    def resetCamera(self, viewer):
        camera = viewer.getCamera()

        camera.setViewport(osg.Viewport(0,0, self.width(), self.height()))

        CAMERA_ANGLE = 45.0
        CAMERA_Z_TRANSLATE = 2.4142135623730949 #1.0 / math.tan(math.radians(CAMERA_ANGLE / 2.0))    
        cameraPosition = [0.0, 0.0, CAMERA_Z_TRANSLATE]

        camera.setProjectionMatrixAsPerspective(CAMERA_ANGLE,float(self.width())/float(self.height()), 1.0, 10000.0)

        eye = osg.Vec3d(cameraPosition[0], cameraPosition[1], cameraPosition[2])
        center = osg.Vec3d(0,0,0)
        up = osg.Vec3d(0,1,0)
        camera.setViewMatrixAsLookAt(eye, center, up)

        camera.getOrCreateStateSet().setAttributeAndModes(osg.BlendFunc(GL.GL_ONE, GL.GL_ONE_MINUS_SRC_ALPHA))
        camera.getOrCreateStateSet().setMode(GL.GL_DEPTH_TEST, False)
        camera.getOrCreateStateSet().setMode(GL.GL_DEPTH_WRITEMASK, False)
        camera.getOrCreateStateSet().setMode(GL.GL_LIGHTING, False)
        material = osg.Material()
        color = osg.Vec4(1.0,1.0,1.0,1.0)
        material.setDiffuse(osg.Material.FRONT_AND_BACK, color)
        material.setAmbient(osg.Material.FRONT_AND_BACK, color)
        camera.getOrCreateStateSet().setAttributeAndModes(material)
        camera.setClearColor(osg.Vec4(0,0,0,0))
        camera.setClearMask(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)

        if not self.gw:
            raise Exception("GraphicsWindow not yet created")
        camera.setGraphicsContext(self.gw)
        self.camera = camera
    
    def heightForWidth(self, w):
        ret = int(w * 9.0 / 16.0)
        return ret
        
    def createViewerNew(self):
        # Standard size
        size = (self.width(), self.height())
        screen_ratio = float(size[0]) / float(size[1])

        # Build the camera
        import math
        CAMERA_ANGLE = 45.0
        CAMERA_Z_TRANSLATE = 2.4142135623730949 #1.0 / math.tan(math.radians(CAMERA_ANGLE / 2.0))
    
        cameraPosition = [0.0, 0.0, CAMERA_Z_TRANSLATE]

        # Create the viewer
        viewer = osgViewer.Viewer()

#        viewer.setSceneData(camera)

        camera = viewer.getCamera()

        # Setup the viewport
        camera.setViewport(osg.Viewport(0,0,size[0], size[1]))
        camera.setReferenceFrame(osg.Transform.ABSOLUTE_RF)
        position = cameraPosition
        camera.setProjectionMatrixAsPerspective(CAMERA_ANGLE, screen_ratio, 0.01, 100.0)
        camera.getOrCreateStateSet().setAttributeAndModes(osg.BlendFunc(GL.GL_ONE, GL.GL_ONE_MINUS_SRC_ALPHA))
        camera.getOrCreateStateSet().setMode(GL.GL_LIGHTING, False)
        material = osg.Material()
        color = osg.Vec4(1.0,1.0,1.0,1.0)
        material.setDiffuse(osg.Material.FRONT_AND_BACK, color)
        material.setAmbient(osg.Material.FRONT_AND_BACK, color)
        camera.getOrCreateStateSet().setAttributeAndModes(material)
        camera.setClearColor(osg.Vec4(0,0,0,0))
        camera.setClearMask(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT)

        eye = osg.Vec3d(cameraPosition[0], cameraPosition[1], cameraPosition[2])
        center = osg.Vec3d(0,0,0)
        up = osg.Vec3d(0,1,0)
        camera.setViewMatrixAsLookAt(eye, center, up)

        self.camera = camera
        
        if not self.gw:
            raise Exception("GraphicsWindow not yet created")

        viewer.getCamera().setGraphicsContext(self.gw)        

        viewer.setThreadingModel(osgViewer.ViewerBase.SingleThreaded)
        
#        viewer.setUpViewInWindow(0,0, size[0], size[1])
#        viewer.addEventHandler(osgViewer.HelpHandler());
#        viewer.addEventHandler(osgViewer.StatsHandler());
        viewer.addEventHandler(osgViewer.ThreadingHandler())
#        viewer.setThreadingModel(osgViewer.ViewerBase.SingleThreaded)

        return viewer

    def setSceneData(self, data):
        self.startTime = time.time()
        if data  != None:
            self.viewer.setSceneData(data)
    
    def getosgviewer(self):
        return self.viewer
    
    def resizeGL( self, w, h ):
        self.gw.resized(0,0,w,h)
        self.resetCamera(self.viewer)

    def paintGL (self):
        t = time.time() - self.startTime
        if t >= self.loopTime:
            self.startTime = time.time()
            t = 0.0

        self.viewer.frameAtTime(t)

    def mousePressEvent( self, event ):
        """put the qt event in the osg event queue"""
        button = mouseButtonDictionary.get(event.button(), 0)
        self.gw.getEventQueue().mouseButtonPress(event.x(), event.y(), button)

    def mouseReleaseEvent( self, event ):
        """put the qt event in the osg event queue"""
        button = mouseButtonDictionary.get(event.button(), 0)
        self.gw.getEventQueue().mouseButtonRelease(event.x(), event.y(), button)

    def mouseMoveEvent(self, event):
        """put the qt event in the osg event queue"""
        self.gw.getEventQueue().mouseMotion(event.x(), event.y())

    def keyPressEvent(self, event):
        """ translate the qt event to an osg event (currently only ascii values) """
        print "key pressed", event.text()
        self.gw.getEventQueue().keyPress( ord(event.text().toAscii().data()) ) #

    def keyReleaseEvent(self, event):
        """ translate the qt event to an osg event (currently only ascii values) """
        print "key released", event.text()
        self.gw.getEventQueue().keyRelease( ord(event.text().toAscii().data()) )

    def timerEvent(self, timerevent):
        """periodically called (each cycle) calls updateGL (which will trigger paintGL)"""
        self.updateGL()

    def getGraphicsWindow(self):
        """returns the osg graphicswindow created by osgViewer """
        return self.gw 


if  __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    mainwindow = MainWindow()

    #show the mainwindow
    mainwindow.show()

    time.sleep(10)
    # create a root node
    node = osg.Group()
    # open a file
    loadedmodel = osgDB.readNodeFile("/Users/lagunas/tmp/osgIphoneExampleGLES2/cow.osg")
    node.addChild(loadedmodel)
    mainwindow.setSceneData(node)

    #execute the application
    sys.exit(app.exec_())



import jarvis
import inspect
from lxml import etree
import os

ml = jarvis.ml

def set_test_module(test_module):
    """Set the module.function that jarvis watches for changes and then executes."""
    # Get the jarvis home directory
    jarvis_home = jarvis.get_home()

    # Find the file that holds the current module.function to test
    testmodulefilename = os.path.join(jarvis_home, jarvis.TEST_MODULE_PATH)

    # Write the file
    f=open(testmodulefilename, "w")
    f.write(test_module)
    f.close()

def error(*args):
    global ml
    if ml != None and ml.display != None:
        ml.display.errorprint(*args)
    
def debug(*args):
    global ml
    if ml != None and ml.display != None:
        ml.display.debugprint(*args)

def debug_dir(object, filt = None):
    debug("debug_dir", object, filt)   
    for k in dir(object):        
        if filt == None or filt in k.lower():
            debug(k)    

def debug_xml(*args):
    debug(*(list(args[:-1]) + [etree.tostring(args[-1], pretty_print = True)]))

def debug_osg(osgdata):
    global ml
    if ml != None and ml.display != None:
        ml.display.osgprint(osgdata)

def debug_osg_set_loop_time(loop_time):
    global ml
    if ml != None and ml.display != None:
        ml.display.setlooptime(loop_time)
        
def testunit_result(result):
    for err in result.errors:
        error(err[1])
    for err in result.failures:
        error(err[1])

def get_osg_viewer():
    global ml
    if ml != None and ml.display != None:
        return ml.display.getosgviewer()

def add_watch_file(filename):
    global ml
    if ml != None:
        return ml.add_watch_file(filename)

def replace_this(*args):
    if len(args) == 0:
        raise Exception("jarvis.commands.replace_this: Nothing to replace by")
    # Retrieve the calling frame
    frame = inspect.stack()[1]
    # Get the file name and the line_number
    file_name = frame[1]
    line_number = frame[2]
    # Split by line the source file
    lines = open(file_name).read().split("\n")
    # Retrieve the right line
    line = lines[line_number - 1]
    # Check that the source code is still matching
    inspect_line = frame[4][0][:-1]
    if line != inspect_line or "replace_this" not in line:
        raise Exception("jarvis.commands.replace_this: looks like the file changed before replace could occur.")

    for i,c in enumerate(inspect_line):
        if c != " ":
            break
    # Build the replacement line
    replacement = " " * i + args[0] % args[1:]
    # Replace the line
    lines[line_number - 1] = replacement
    # Rebuild the file content
    file_content = "\n".join(lines)
    # Write back the file
    file = open(file_name, "w")
    file.write(file_content)
    file.close()    
